#define BOOST_TEST_DYN_LINK
#define BOOST_TEST_MODULE 0003
#define BOOST_TEST_NO_MAIN

#include <boost/test/unit_test.hpp>
#include <boost/lexical_cast.hpp>
#include <AMDiS.h>

using namespace AMDiS;
using namespace std;
using namespace AMDiS::Parallel;
using boost::lexical_cast;

/*
  - Test 2-level FETI-DP on the number of global primal and dual nodes.
*/

BOOST_AUTO_TEST_CASE(amdis_mpi_feti_multilevel)
{
  BOOST_REQUIRE(MPI::COMM_WORLD.Get_size() == 16);

  ProblemStat ellipt("ellipt");
  ellipt.initialize(INIT_ALL);

  MeshDistributor *meshDist = MeshDistributor::globalMeshDistributor;
  meshDist->initParallelization();

  Mesh* mesh = ellipt.getMesh();
  BOOST_REQUIRE(mesh->getNumberOfLeaves() == 8);
  BOOST_REQUIRE(meshDist->getMeshLevelData().getLevelNumber() == 2);

  vector<const FiniteElemSpace*> feSpaces;
  feSpaces.push_back(ellipt.getFeSpace(0));

  PetscSolverFeti &feti = *(dynamic_cast<PetscSolverFeti*>(ellipt.getPetscSolver()));
  feti.initialize();
  feti.createFetiData();

  if (feti.getNumberOfPrimals() != 5) {
    BOOST_ERROR("Number of primals is " + lexical_cast<string>(feti.getNumberOfPrimals()) + " and not 5!\n");
  }

  if (feti.getNumberOfDuals() != 24) {
    BOOST_ERROR("Number of duals is " + lexical_cast<string>(feti.getNumberOfDuals()) + " and not 24!\n");
  }

  if (feti.getNumberOfLagrange() != 12) {
    BOOST_ERROR("Number of lagrange constraints is " + lexical_cast<string>(feti.getNumberOfLagrange()) + " and not 24!\n");
  }

  ParallelDofMapping &dofMapSd = ellipt.getPetscSolver()->getDofMapSd();

  if (dofMapSd.getOverallDofs() != 25) {
    BOOST_ERROR("Number of subdomain DOFs is " + lexical_cast<string>(dofMapSd.getOverallDofs()) + " and not 25!\n");
  }

  DofComm& commSd = meshDist->getDofCommSd();

  if (commSd.getNumberDofs(commSd.getSendDofs(), 0, feSpaces[0]) + 
      commSd.getNumberDofs(commSd.getRecvDofs(), 0, feSpaces[0]) != 5) {
    BOOST_ERROR("Something wrong with subdomain communicator!\n");
  }

  int sum = 
    commSd.getNumberDofs(commSd.getSendDofs(), 0, feSpaces[0], true) + 
    commSd.getNumberDofs(commSd.getRecvDofs(), 0, feSpaces[0], true);

  if (meshDist->getMeshLevelData().getMpiComm(1).Get_rank() == 3) {
    BOOST_CHECK(sum == 7);
  } else {
    BOOST_CHECK(sum == 5);
  }
}


int main(int argc, char **argv)
{
  AMDiS::init(argc, argv, "./init/test0003.dat.2d");

  boost::unit_test::unit_test_main(&init_unit_test, argc, argv);

  AMDiS::finalize();
}
